@@ -31,7 +31,7 @@ const WorldContentView = new Lang.Class({
     Name: 'WorldContentView',
     Extends: Gtk.Popover,
 
-    _init: function(application, params) {
+    _init: function(application, window, params) {
         params = Params.fill(params, { hexpand: false, vexpand: false });
         this.parent(params);
 
@@ -44,6 +44,7 @@ const WorldContentView = new Lang.Class({
         this.add(grid);
 
         this.model = application.model;
+        this._window = window;
 
         this._listbox = builder.get_object('locations-list-box');
         this._listbox.set_header_func(function (row, previous) {
@@ -86,12 +87,13 @@ const WorldContentView = new Lang.Class({
 
         this._listbox.connect('row-activated', Lang.bind(this, function(listbox, row) {
             this.hide();
-            this.model.showInfo(row._info, row._isCurrentLocation);
+            this.model.moveLocationToFront(row._info);
+            this._window.showInfo(row._info, false);
         }));
 
-        this.model.connect('current-location-changed', Lang.bind(this, function(model, location) {
+        this.model.connect('current-location-changed', Lang.bind(this, function(model, info) {
             autoLocStack.visible_child_name = 'auto-location-switch-grid';
-            if (location) {
+            if (info.location) {
                 GObject.signal_handler_block(autoLocSwitch, handlerId);
                 autoLocSwitch.active = true;
                 GObject.signal_handler_unblock(autoLocSwitch, handlerId);
@@ -102,34 +104,46 @@ const WorldContentView = new Lang.Class({
 
                 autoLocSwitch.sensitive = false;
             }
+
+            this._window.showInfo(info, true);
         }));
 
-        let stackPopover = builder.get_object('popover-stack');
-        this.model.connect('revalidate', Lang.bind(this, function() {
-            this._listbox.invalidate_filter();
+        this._stackPopover = builder.get_object('popover-stack');
+        this._listbox.set_filter_func(Lang.bind(this, this._filterListbox));
 
-            let children = this._listbox.get_children();
-            if (children.length == 1)
-                stackPopover.set_visible_child_name("search-grid");
-            else
-                stackPopover.set_visible_child_name("locations-grid");
-        }));
-
-        this._listbox.set_filter_func(Lang.bind(this, this._filterListbox, this.model));
         this.model.connect('location-added', Lang.bind(this, this._onLocationAdded));
         this.model.connect('location-removed', Lang.bind(this, this._onLocationRemoved));
 
-        this.model.load();
+        this._currentLocationAdded = false;
+        if (this.model.length > 0) {
+            this.model.getAll().forEach(Lang.bind(this, function(info) {
+                this._onLocationAdded(this.model, info, info._isCurrentLocation);
+            }));
+        } else {
+            this.model.load();
+        }
     },
 
-    _filterListbox: function(row, model) {
-        return model.currentlyLoadedInfo == null ||
-            row._info != model.currentlyLoadedInfo;
+    refilter: function() {
+        this._listbox.invalidate_filter();
+    },
+
+    _syncStackPopover: function() {
+        if (this.model.length == 1)
+            this._stackPopover.set_visible_child_name("search-grid");
+        else
+            this._stackPopover.set_visible_child_name("locations-grid");
+    },
+
+    _filterListbox: function(row) {
+        return this._window.currentInfo == null ||
+            row._info != this._window.currentInfo;
     },
 
     _locationChanged: function(entry) {
         if (entry.location) {
-            this.model.addNewLocation(entry.location, false);
+            let info = this.model.addNewLocation(entry.location, false);
+            this._window.showInfo(info, false);
             this.hide();
             entry.location = null;
         }
@@ -183,13 +197,16 @@ const WorldContentView = new Lang.Class({
         row._isCurrentLocation = isCurrentLocation;
 
         if (isCurrentLocation) {
-            if (model.addedCurrentLocation) {
-                this._listbox.get_row_at_index(0).destroy();
+            if (this._currentLocationAdded) {
+                let row0 = this._listbox.get_row_at_index(0);
+                if (row0)
+                    row0.destroy();
             }
 
+            this._currentLocationAdded = true;
             this._listbox.insert(row, 0);
         } else {
-            if (model.addedCurrentLocation)
+            if (this._currentLocationAdded)
                 this._listbox.insert(row, 1);
             else
                 this._listbox.insert(row, 0);
@@ -202,6 +219,8 @@ const WorldContentView = new Lang.Class({
             tempLabel.label = info.get_temp_summary();
             image.icon_name = info.get_symbolic_icon_name();
         }));
+
+        this._syncStackPopover();
     },
 
     _onLocationRemoved: function(model, info) {
@@ -218,5 +237,9 @@ const WorldContentView = new Lang.Class({
             info.disconnect(info._updatedId);
             info._updatedId = 0;
         }
+        if (info._isCurrentLocation)
+            this._currentLocationAdded = false;
+
+        this._syncStackPopover();
     },
 });
